/*
 * Title:		wkStreetCountdown
 * Author:		Milord (AKA Jengu)
 * Date:		28/11/2017 (DD/MM/YYYY)
 * Version:		1.0
 * Description:	A Worms Armageddon WormKit module that optionally shows mine and weapon fuses.
 */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <shlwapi.h>
#include <string.h>
#include <wchar.h>
#include <stdint.h>

BOOL MemoryPatch(void *dest, const void *src, size_t count, DWORD flNewProtect) // Combines memcpy() and VirtualProtect().
{
	DWORD flOldProtect;
	
	if(!(VirtualProtect(dest, count, flNewProtect, &flOldProtect)))
	{
		return FALSE;
	}
	
	memcpy(dest, src, count);
	
	if(!(VirtualProtect(dest, count, flOldProtect, &flNewProtect)))
	{
		return FALSE; // This function only reports whether or not VirtualProtect() failed at some point.
	}
	
	return TRUE;
}

void *GetSubstringPointer(const void *string, size_t stringLength, const void *substring, size_t substringLength)
{
	for(int i = 0; i <= stringLength - substringLength; i++)
	{
		for(int j = 0; ((uint8_t*) string)[i + j] == ((uint8_t*) substring)[j]; j++)
		{
			if(j == substringLength - 1)
			{
				return (void*) &((uint8_t*) string)[i];
			}
		}
	}
	
	return NULL;
}

void CreateDefaultIni()
{
	if(!(WritePrivateProfileStringW(L"wkStreetCountdown", L"ShowMineFuses", L"1", L".\\wkStreetCountdown.ini")))
	{
		;
	}
	
	if(!(WritePrivateProfileStringW(L"wkStreetCountdown", L"ShowWeaponFuses", L"1", L".\\wkStreetCountdown.ini")))
	{
		;
	}
}

DWORD WINAPI ModuleMain(LPVOID lpParameter) // ThreadProc().
{
	const uint8_t MINE_FUSE_SIGNATURE[] = {0x04, 0x8B, 0xE9, 0xEB};
	const int MINE_FUSE_OFFSET = -28;
	const uint8_t MINE_FUSE_PATCH[] = {0x82};
	
	const uint8_t WEAPON_FUSE_SIGNATURE[] = {0x24, 0x7D, 0x10, 0x8B};
	const int WEAPON_FUSE_OFFSET = 13;
	const uint8_t WEAPON_FUSE_PATCH[] = {0x72};
	
	HMODULE baseAddress;
	
	if(!(PathFileExistsW(L".\\wkStreetCountdown.ini")))
	{
		CreateDefaultIni();
	}
	
	if(!(baseAddress = GetModuleHandle(NULL)))
	{
		return 0; // Could not get the base memory address.
	}
	
	{
		wchar_t lpReturnedString[128]; // 128 characters will be more than enough.
		
		if(GetPrivateProfileStringW(L"wkStreetCountdown", L"ShowMineFuses", NULL, lpReturnedString, sizeof(lpReturnedString), L".\\wkStreetCountdown.ini"))
		{
			if(!(wcscmp(lpReturnedString, L"1"))) // wcscmp() returns 0 if both strings are equal.
			{
				void *p;
				
				if((p = GetSubstringPointer(baseAddress, 0x200000, MINE_FUSE_SIGNATURE, sizeof(MINE_FUSE_SIGNATURE))))
				{
					if(!(MemoryPatch((uint8_t*) p + MINE_FUSE_OFFSET, MINE_FUSE_PATCH, sizeof(MINE_FUSE_PATCH), PAGE_EXECUTE_READWRITE)))
					{
						;
					}
				}
			}
		}
	}
	
	{
		wchar_t lpReturnedString[128]; // 128 characters will be more than enough.
		
		if(GetPrivateProfileStringW(L"wkStreetCountdown", L"ShowWeaponFuses", NULL, lpReturnedString, sizeof(lpReturnedString), L".\\wkStreetCountdown.ini"))
		{
			if(!(wcscmp(lpReturnedString, L"1"))) // wcscmp() returns 0 if both strings are equal.
			{
				void *p;
				
				if((p = GetSubstringPointer(baseAddress, 0x200000, WEAPON_FUSE_SIGNATURE, sizeof(WEAPON_FUSE_SIGNATURE))))
				{
					if(!(MemoryPatch((uint8_t*) p + WEAPON_FUSE_OFFSET, WEAPON_FUSE_PATCH, sizeof(WEAPON_FUSE_PATCH), PAGE_EXECUTE_READWRITE)))
					{
						;
					}
				}
			}
		}
	}
	
	return 1;
}

BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	switch(fdwReason)
	{
		case DLL_PROCESS_ATTACH:
			DisableThreadLibraryCalls(hinstDLL);
			HANDLE hObject;
			if(!(hObject = CreateThread(NULL, 0, ModuleMain, NULL, 0, NULL)))
			{
				return FALSE; // CreateThread() failed.
			}
			CloseHandle(hObject);
			break;
	}
	return TRUE;
}